library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use WORK.leddcd_pckg.all;

entity ramsum is
	port (
		rst	: in STD_LOGIC;											-- reset
		clk	: in STD_LOGIC;											-- clock
		s		: out STD_LOGIC_VECTOR(6 downto 0)	-- outputs to LED segments
	);
end ramsum;

architecture ramsum_arch of ramsum is
component ram	-- 16-byte synchronous RAM from CoreGen
	port (
		a		: IN std_logic_VECTOR(3 downto 0);	-- address bus
		d		: IN std_logic_VECTOR(7 downto 0);	-- data input bus
		we	: IN std_logic;											-- write-enable
		c		: IN std_logic;											-- clock
		ce	: IN std_logic;											-- clock enable
		q		: OUT std_logic_VECTOR(7 downto 0)	-- data output bus
	);
end component;
-- RAM address, data, control signals
signal		addr_r, next_addr	: UNSIGNED(3 downto 0);	-- RAM address register
signal		din		: UNSIGNED(7 downto 0);			-- RAM data input bus
signal		dout	: UNSIGNED(7 downto 0);			-- RAM data output bus
signal		ce		: STD_LOGIC;								-- RAM chip-enable
signal		we		: STD_LOGIC;								-- RAM write-enable
signal		aa		: STD_LOGIC_VECTOR(addr_r'range);		-- RAM address bus
signal		dd,qq	: STD_LOGIC_VECTOR(din'range);			-- RAM data in/out buses
constant	maxaddr	: UNSIGNED := TO_UNSIGNED(10,addr_r'length);	-- upper address
signal		sum_r, next_sum	: UNSIGNED(din'length-1 downto 0);	-- stores sum of RAM bytes
signal		delay_r, next_delay	: UNSIGNED(22 downto 0);	-- delay counter
constant	blank_dly						: UNSIGNED := TO_UNSIGNED(5_000_000,delay_r'length);
constant	interdigit_dly			: UNSIGNED := TO_UNSIGNED(1_600_000,delay_r'length);
constant	digit_dly						: UNSIGNED := TO_UNSIGNED(2_500_000,delay_r'length);
signal		digit	: UNSIGNED(3 downto 0);	-- LED hex digit to display
signal		blank	: STD_LOGIC;	-- LED digit blanking signal
-- states for the state machine
type state is (init,invertr,invertw,add,display_blank,
			display_upper_digit,display_interdigit,display_lower_digit);
signal		st_r, next_st	: state;		-- state register
begin

-- this process computes the actions of the state machine in each state
process(clk,st_r,addr_r,sum_r,delay_r,din)
begin
	-- default outputs unless otherwise specified
	next_st			<= st_r;			-- next state is the same as the current state
	next_addr		<= addr_r;		-- don't change the RAM address
	next_sum		<= sum_r;			-- don't update the sum register
	next_delay	<= delay_r-1;	-- decrement the delay counter
	digit				<= TO_UNSIGNED(0,digit'length);	-- output a '0' LED digit
	blank				<= '1';				-- blank the LED display
	ce					<= '1';				-- always read the RAM
	we					<= '0';				-- don't write to the RAM
	din					<= TO_UNSIGNED(0,din'length); 
	
	case st_r is	-- case statement for the state machine
	when init =>	-- initialization state
		next_addr <= maxaddr;	-- start inverting from the upper address
		next_st <= invertw;		-- enter the RAM inversion loop
	when invertw =>	-- write inverted byte value into same RAM location
		we <= '1';						-- write RAM at the end of this clock cycle
		din <= TO_UNSIGNED(0,din'length) - dout;	-- output inverted byte value to RAM
		next_st <= invertr;		-- now read from next RAM location
	when invertr =>	-- read byte from RAM
		if addr_r = TO_UNSIGNED(0,addr_r'length) then
			-- reached the lower address of the RAM data
			next_addr <= maxaddr;						-- reload register with upper address
			next_st <= add;									-- enter the summation loop
		else
			-- haven't inverted all the RAM data yet
			next_addr <= addr_r - 1;
			next_st <= invertw;							-- now write to it
		end if;
	when add =>		-- sum the inverted data from RAM
		next_sum <= sum_r + dout;	-- add the RAM data to the sum
		if addr_r = TO_UNSIGNED(0,addr_r'length) then
			-- reached the lower address of the RAM data
			next_delay <= blank_dly;				-- load display interval counter
			next_st <= display_blank;				-- now display the sum
		else
			-- haven't summed all the RAM data yet so stay in this state
			next_addr <= addr_r - 1;				-- address the next RAM location
		end if;
	when display_blank =>	-- blank the display
		if delay_r = TO_UNSIGNED(0,delay_r'length) then
			-- initial display blanking is complete
			next_delay <= digit_dly;				-- load digit display interval
			next_st <= display_upper_digit;	-- now display the upper sum digit
		end if;
	when display_upper_digit => -- display the upper digit of the sum
		blank <= '0';											-- activate the LED
		digit <= sum_r(7 downto 4);				-- display the upper 4-bits of the sum
		if delay_r = TO_UNSIGNED(0,delay_r'length) then
			-- upper digit display is complete
			next_delay <= interdigit_dly;		-- load inter-digit blanking interval
			next_st <= display_interdigit;	-- blank the display between digits
		end if;
	when display_interdigit =>	-- blank the display between sum digits
		if delay_r = TO_UNSIGNED(0,delay_r'length) then
			-- inter-digit display blanking is complete
			next_delay <= digit_dly;				-- load digit display interval
			next_st <= display_lower_digit;	-- now display the lower sum digit
		end if;
	when display_lower_digit => -- display the lower digit of the sum
		blank <= '0';											-- activate the LED
		digit <= sum_r(3 downto 0);				-- display the lower 4-bits of the sum
		if delay_r = TO_UNSIGNED(0,delay_r'length) then
			-- lower digit display is complete
			next_delay <= blank_dly;				-- load blanking interval between loops
			next_st <= display_blank;				-- loop and display the sum again
		end if;
	when others =>	-- error state
		next_st <= init;	-- re-initialize the state machine on an erroneous state
	end case;
end process;

-- this process updates the registers on every rising clock edge
process(clk)
begin
	if clk'event and clk='1' then	-- trigger on rising clock edge
		if rst='1' then		-- synchronous reset
			st_r		<= init;
			sum_r		<= TO_UNSIGNED(0,sum_r'length);
		else							-- update the registers
			st_r		<= next_st;
			sum_r		<= next_sum;
			addr_r	<= next_addr;
			delay_r	<= next_delay;
		end if;
	end if;
end process;

-- connect clock, address, data and control to the RAM block
aa <= STD_LOGIC_VECTOR(next_addr);
dd <= STD_LOGIC_VECTOR(din);
qq <= STD_LOGIC_VECTOR(dout);
u2: ram	port map (
			a => aa,
			d => dd,
			we => we,
			c => clk,
			ce => ce,
			q => qq
			);

-- output digit on the LED display
u1: leddcd port map(blank=>blank, d=>digit, s=>s);

end ramsum_arch;
